macro_rules! stream {
($($data:expr),* $(,)? => $dest:expr) => { ... };
}Expand description
Writes all arguments to a byte buffer, taking each as a reference.
macro_rules! stream {
($($data:expr),* $(,)? => $dest:expr) => { ... };
}Writes all arguments to a byte buffer, taking each as a reference.