macro_rules! write_le_num {
($T:ty; $dest:expr, $num:expr) => { ... };
}Expand description
Writes a numeric value to the buffer in little-endian format.
macro_rules! write_le_num {
($T:ty; $dest:expr, $num:expr) => { ... };
}Writes a numeric value to the buffer in little-endian format.