Macros
- vec8
- Initializes a vector with an 8-bit size.
- vec16
- Initializes a vector with an 16-bit size.
- vec32
- Initializes a vector with an 32-bit size.
- vec64
- Initializes a vector with an 64-bit size.
- vec128
- Initializes a vector with an 128-bit size.
- write_
all - Writes all provided data to the destination.
- write_
le_ num - Writes a numeric value to the buffer in little-endian format.
Structs
Enums
- Decode
Error - Denotes a fatal error during decoding.
- Encode
Error - Denotes a fatal error during encoding.
Traits
- CharExt
- Read
- Serialize
- A value that can be serialized to and deserialized from a byte buffer.
- Slice
Ext - ToTape
- Write
Type Aliases
- String
Size - The type used to represent the length of a string when serialized.