stream

Macro stream 

macro_rules! stream {
    ($($data:expr),* $(,)? => $dest:expr) => { ... };
}
Writes all provided data to the destination, taking each argument as a reference.

This macro is useful for streaming known sequences of values to a destination.